/**
 * This version of the parallel prime-check program uses an extra thread to
 * monitor the progress of the primality-checking thread. It shows how to use
 * a Boolean variable, called "done" here, to signal to the progress-checking
 * thread that it's time to exit.
 */
public class C_Progress {
  public static void main(String[] args) {
    PrimeCheckP check = new PrimeCheckP(93877037L);

    Thread checkThread = new Thread(check);
    Thread progressThread = new Thread(new Progress(check));

    progressThread.start();
    checkThread.start();
  }
}

class PrimeCheckP implements Runnable {
  private long num;  // The number to check.
  public long i;  // The divisor we're currently checking.
  public boolean done;  // Are we done checking the number?

  public PrimeCheckP(long n) {
    this.num = n;
    i = 0;
    done = false;
  }

  public void run() {
    for (i = 2; i < num; ++i) {
      if (num % i == 0) {
        System.out.println(num + " is composite: divisible by " + i);
        done = true;
        return;
      }
    }
    System.out.println(num + " is prime!");
    done = true;
  }
}

/**
 * Periodically check on the progress of a PrimeCheckP, printing out the
 * largest divisor it has checked so far.
 */
class Progress implements Runnable {
  private PrimeCheckP check;

  /**
   * Create a progress monitor for a given prime check.
   */
  public Progress(PrimeCheckP check) {
    this.check = check;
  }

  /**
   * Periodically print the largest divisor tested in the prime check.
   */
  public void run() {
    while (true) {
      // Wait for a little while between print calls.
      try {
        Thread.sleep(100);
      } catch (InterruptedException e) { }

      // Exit the thread when the number check is done.
      if (check.done) {
        break;
      }

      System.out.println("checked up to " + check.i + "...");
    }
  }
}
