/**
 * This example, like the last one, monitors the progress of a primality
 * check, but it does not use a Boolean variable to tell the progress thread
 * to shut down. Instead, it marks the thread as a "daemon", which means it
 * automatically exits when the rest of the threads (i.e., the prime-checking
 * thread and the main thread) exit.
 */
public class D_Progress_Daemon {
  public static void main(String[] args) {
    PrimeCheckD check = new PrimeCheckD(93877037L);

    Thread checkThread = new Thread(check);
    Thread progressThread = new Thread(new ProgressD(check));

    // Mark the progress thread as a "daemon", so it automatically exits when
    // all the non-daemon threads in the process exit.
    progressThread.setDaemon(true);

    progressThread.start();
    checkThread.start();
  }
}

class PrimeCheckD implements Runnable {
  private long num;  // The number to check.
  public long i;  // The divisor we're currently checking.

  public PrimeCheckD(long n) {
    this.num = n;
    i = 0;
  }

  public void run() {
    for (i = 2; i < num; ++i) {
      if (num % i == 0) {
        System.out.println(num + " is composite: divisible by " + i);
        return;
      }
    }
    System.out.println(num + " is prime!");
  }
}

/**
 * Periodically check on the progress of a PrimeCheckP, printing out the
 * largest divisor it has checked so far.
 */
class ProgressD implements Runnable {
  private PrimeCheckD check;

  /**
   * Create a progress monitor for a given prime check.
   */
  public ProgressD(PrimeCheckD check) {
    this.check = check;
  }

  /**
   * Periodically print the largest divisor tested in the prime check.
   */
  public void run() {
    while (true) {
      // Wait for a little while between print calls.
      try {
        Thread.sleep(100);
      } catch (InterruptedException e) { }

      System.out.println("checked up to " + check.i + "...");
    }
  }
}
